#! /usr/bin/python
# coding=utf-8

# Copyright 2023 teapot
#
# Licensed under the Apache Licence, Version 2.0 (the "Licence");
# you may not use this file except in compliance with the Licence.
# You may obtain a copy of the Licence at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# For the reason for writing this plugin, see my post:
#     http://gimpchat.com/viewtopic.php?f=11&t=18573&start=890#p280490

# Version 2.0.0

from gimpfu import *

PARENT_NONE = None
POSITION_TOP = 0

def init():
    blurb = '''Make frames that can be used for an animation by repeatedly copying the input layer and changing the hue by 360 / number of frames each time.

Warning: The input layer's position in the layer stack is put in sequence.'''

    register(
        'plug-in-tp-rotate-hue',                   # Name
        blurb,                                     # Blurb
        blurb,                                     # Help
        'teapot',                                  # Author
        'teapot',                                  # Copyright holder
        '2023',                                    # Date
        'Rotate Hue...',                           # Menu name
        'RGB*,INDEXED*',                           # Image types
        [                                          # Parameters
            (PF_IMAGE, 'image', 'Input image', None),
            (PF_DRAWABLE, 'drawable', 'Input drawable', None),
            (PF_INT, 'frames', 'Number of frames', 18),
            (PF_BOOL, 'reverse', 'Reverse order?', False),
        ],
        [                                          # Results: [(type, name, description), ...]
        ],
        callback,                                  # Callback function
        menu='<Image>/Filters/Animation/Animators' # Menu path
    )

def callback(image, drawable, frames, reverse):
    if not pdb.gimp_item_is_layer(drawable):
        msg('Want a layer.')
        return

    if pdb.gimp_item_is_group(drawable):
        msg("Doesn't work on a layer group.")
        return

    if frames < 1:
        msg('Want at least 1 frame, got %s' % frames)
        return

    pdb.gimp_image_undo_group_start(image)

    try:
        rotate_hue(image, drawable, frames, reverse)

    except Exception:
        import traceback
        msg(traceback.format_exc())

    finally:
        pdb.gimp_image_undo_group_end(image)

def rotate_hue(image, layer, frames, reverse):
    hue_offset = 0 # float -180 <= hue_offset <= 180
    lightness  = 0 # float -100 <= lightness  <= 100
    saturation = 0 # float -100 <= saturation <= 100
    overlap    = 0 # float    0 <= overlap    <= 100

    original_name = layer.name
    layer.name = '%s hue 0.00' % original_name
    step = 360.0 / frames

    for frame in range(1, frames):
        hue_offset = frame * step
        if hue_offset > 180:
            hue_offset -= 360
        if reverse:
            hue_offset = -hue_offset

        new_layer = layer.copy()
        new_layer.name = '%s hue %.2f' % (original_name, hue_offset)
        pdb.gimp_image_insert_layer(image, new_layer, PARENT_NONE, POSITION_TOP)

        pdb.gimp_drawable_hue_saturation(new_layer, HUE_RANGE_ALL, hue_offset, lightness, saturation, overlap)

    if reverse:
        position = POSITION_TOP
    else:
        position = frames - 1
    pdb.gimp_image_reorder_item(image, layer, PARENT_NONE, position)

def msg(s):
    pdb.gimp_message('%s' % s)

init()
main()
